import { __spread, __values, __decorate, __metadata } from 'tslib';
import { CommonModule } from '@angular/common';
import { Input, Directive, EventEmitter, ElementRef, ChangeDetectorRef, Output, ContentChildren, QueryList, HostListener, Component, ViewEncapsulation, ViewChild, TemplateRef, HostBinding, ComponentFactoryResolver, ViewContainerRef, NgZone, ContentChild, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

var styleProperties = Object.freeze([
    'direction',
    'boxSizing',
    'width',
    'height', 'minHeight', 'minWidth', 'overflowX',
    'overflowY',
    'borderTopWidth', 'borderRightWidth', 'borderBottomWidth', 'borderLeftWidth', 'borderStyle',
    'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft',
    // https://developer.mozilla.org/en-US/docs/Web/CSS/font
    'fontStyle', 'fontVariant', 'fontWeight', 'fontStretch', 'fontSize', 'fontSizeAdjust', 'lineHeight', 'fontFamily',
    'textAlign', 'textTransform', 'textIndent',
    'textDecoration',
    'letterSpacing', 'wordSpacing',
    'tabSize', 'MozTabSize'
]);
var isBrowser = typeof window !== 'undefined';
var isFirefox = isBrowser && window.mozInnerScreenX != null;
function isInputOrTextAreaElement(element) {
    return element !== null &&
        (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA' || element.nodeName === '#text');
}
function spliceString(value, start, end, insert) {
    return value.substring(0, start) + insert + value.substring(end) + '';
}
function iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform) {
    var match, start = 0, currentPlainTextIndex = 0;
    var regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var display = displayTransform.apply(void 0, __spread(match));
        var substr = value.substring(start, match.index);
        textIterator(substr, start, currentPlainTextIndex);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
    if (start < value.length) {
        textIterator(value.substring(start), start, currentPlainTextIndex);
    }
}
function iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform) {
    var match, start = 0, currentPlainTextIndex = 0;
    var regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var display = displayTransform.apply(void 0, __spread(match));
        var substr = value.substring(start, match.index);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
}
function mapPlainTextIndex(value, mentionMarkup, indexInPlainText, toEndOfMarkup, displayTransform) {
    if (isNaN(indexInPlainText)) {
        return indexInPlainText;
    }
    var result;
    var textIterator = function (matchString, index, substringPlainTextIndex) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (substringPlainTextIndex + matchString.length >= indexInPlainText) {
            result = index + indexInPlainText - substringPlainTextIndex;
        }
    };
    var markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (mentionPlainTextIndex + display.length > indexInPlainText) {
            result = index + (toEndOfMarkup ? matchString.length : 0);
        }
    };
    iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform);
    return typeof result !== 'undefined' ? result : value.length;
}
function getCaretPosition(element) {
    if (isInputOrTextAreaElement(element)) {
        var value = element.value;
        return value.slice(0, element.selectionStart).length;
    }
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
        var range = selection.getRangeAt(0);
        return range.startOffset;
    }
    return 0;
}
function getCaretCoordinates(element, position) {
    var coords = { top: 0, left: 0 };
    if (!isBrowser) {
        return coords;
    }
    var div = document.createElement('div');
    document.body.appendChild(div);
    var style = div.style;
    var computed = getElementStyle(element);
    style.whiteSpace = 'pre-wrap';
    if (element.nodeName !== 'INPUT') {
        style.wordWrap = 'break-word';
    }
    style.position = 'absolute';
    style.visibility = 'hidden';
    styleProperties.forEach(function (prop) { return style[prop] = computed[prop]; });
    if (isFirefox) {
        if (element.scrollHeight > parseInt(computed.height, 10)) {
            style.overflowY = 'scroll';
        }
    }
    else {
        style.overflow = 'hidden';
    }
    div.textContent = element.value.substring(0, position);
    if (element.nodeName === 'INPUT') {
        div.textContent = div.textContent.replace(/\s/g, '\u00a0');
    }
    var span = document.createElement('span');
    span.textContent = element.value.substring(position) || '.';
    div.appendChild(span);
    var scrollTop = 0;
    if (element.scrollTop > 0) {
        scrollTop = element.scrollTop;
    }
    coords = {
        top: span.offsetTop + parseInt(computed['borderTopWidth'], 10) - scrollTop,
        left: span.offsetLeft + parseInt(computed['borderLeftWidth'], 10)
    };
    document.body.removeChild(div);
    return coords;
}
function getElementStyle(element, property) {
    var style = window.getComputedStyle ? getComputedStyle(element) : element.currentStyle;
    if (property && typeof property === 'string' && typeof style[property] !== 'undefined') {
        return style[property];
    }
    else if (property && typeof property === 'string') {
        return null;
    }
    return style;
}
function setCaretPosition(element, position) {
    if (isInputOrTextAreaElement(element) && element.selectionStart !== undefined) {
        element.focus();
        element.setSelectionRange(position, position);
    }
    else {
        var range = document.createRange();
        range.setStart(element, position);
        range.collapse(true);
        var selection = window.getSelection();
        selection.removeAllRanges();
        selection.addRange(range);
    }
}
function escapeRegExp(str) {
    return str.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}
function markupToRegExp(markup) {
    var placeholderRegExp = /__([\w]+)__/g;
    var placeholderExclusion = '^\\)\\]';
    var markupPattern = escapeRegExp(markup);
    var placeholders = {};
    var match, i = 1;
    do {
        match = placeholderRegExp.exec(markupPattern);
        if (match) {
            var placeholder = match[1];
            markupPattern = markupPattern.replace("__" + placeholder + "__", "([" + placeholderExclusion + "]+)");
            placeholders[placeholder] = ++i;
        }
    } while (match);
    return { markup: markup, regEx: new RegExp('(' + markupPattern + ')', 'ig'), groups: placeholders };
}
function getPlainText(value, mentionMarkup, displayTransform) {
    mentionMarkup.regEx.lastIndex = 0;
    return value.replace(mentionMarkup.regEx, displayTransform);
}
function replacePlaceholders(item, markupMention) {
    var result = markupMention.markup + '';
    Object.keys(markupMention.groups)
        .forEach(function (key) { return result = result.replace(new RegExp("__" + key + "__", 'g'), typeof item === 'string' ? item : item[key]); });
    return result;
}
function applyChangeToValue(value, markupMention, plainTextValue, selectionStartBeforeChange, selectionEndBeforeChange, selectionEndAfterChange, displayTransform) {
    if (selectionStartBeforeChange === void 0) { selectionStartBeforeChange = 0; }
    if (selectionEndBeforeChange === void 0) { selectionEndBeforeChange = 0; }
    if (selectionEndAfterChange === void 0) { selectionEndAfterChange = 0; }
    var oldPlainTextValue = getPlainText(value, markupMention, displayTransform);
    var lengthDelta = oldPlainTextValue.length - plainTextValue.length;
    if (!selectionStartBeforeChange) {
        selectionStartBeforeChange = selectionEndBeforeChange + lengthDelta;
    }
    if (!selectionEndBeforeChange) {
        selectionEndBeforeChange = selectionStartBeforeChange;
    }
    if (selectionStartBeforeChange === selectionEndBeforeChange && selectionEndBeforeChange === selectionEndAfterChange &&
        oldPlainTextValue.length === plainTextValue.length) {
        selectionStartBeforeChange--;
    }
    var insert = plainTextValue.slice(selectionStartBeforeChange, selectionEndAfterChange);
    var spliceStart = Math.min(selectionStartBeforeChange, selectionEndAfterChange);
    var spliceEnd = selectionEndBeforeChange;
    if (selectionStartBeforeChange === selectionEndAfterChange) {
        spliceEnd = Math.max(selectionEndBeforeChange, selectionStartBeforeChange + lengthDelta);
    }
    return spliceString(value, mapPlainTextIndex(value, markupMention, spliceStart, false, displayTransform), mapPlainTextIndex(value, markupMention, spliceEnd, true, displayTransform), insert);
}
function findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform) {
    var result = { start: -1, end: -1 };
    var markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (mentionPlainTextIndex < indexInPlainText && mentionPlainTextIndex + display.length > indexInPlainText) {
            result = { start: mentionPlainTextIndex, end: mentionPlainTextIndex + display.length };
            return true;
        }
        return false;
    };
    iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform);
    return result;
}
function getBoundsOfMentionAtPosition(value, mentionMarkup, indexInPlainText, displayTransform) {
    return findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform);
}
function escapeHtml(text) {
    return text.replace(/</g, '&lt;').replace(/>/g, '&gt;');
}
function isCoordinateWithinRect(rect, x, y) {
    return (rect.left < x && x < rect.right) && (rect.top < y && y < rect.bottom);
}
var Highlighted = /** @class */ (function () {
    function Highlighted(element, type) {
        if (type === void 0) { type = null; }
        this.element = element;
        this.type = type;
    }
    Object.defineProperty(Highlighted.prototype, "clientRect", {
        get: function () {
            return this.element.getBoundingClientRect();
        },
        enumerable: true,
        configurable: true
    });
    return Highlighted;
}());

/**
 * The Highlighted Pattern Directive
 */
var NgHighlighterPatternDirective = /** @class */ (function () {
    function NgHighlighterPatternDirective() {
        var _this = this;
        this.format = function (content) {
            var e_1, _a;
            if (typeof _this.markupReplace === 'string') {
                var result = void 0;
                var replaceTries = [_this.markupReplace, "$" + _this.markupReplace, "$" + _this.markupMention.groups[_this.markupReplace]];
                try {
                    for (var replaceTries_1 = __values(replaceTries), replaceTries_1_1 = replaceTries_1.next(); !replaceTries_1_1.done; replaceTries_1_1 = replaceTries_1.next()) {
                        var attempt = replaceTries_1_1.value;
                        result = content.replace(_this.markupMention.regEx, attempt);
                        if (result !== attempt) {
                            break;
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (replaceTries_1_1 && !replaceTries_1_1.done && (_a = replaceTries_1.return)) _a.call(replaceTries_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return result;
            }
            else if (typeof _this.markupReplace === 'function') {
                return _this.markupReplace(content);
            }
            return content;
        };
    }
    NgHighlighterPatternDirective.prototype.ngOnInit = function () {
        if (this.markup) {
            this.markupMention = markupToRegExp(this.markup);
        }
    };
    NgHighlighterPatternDirective.prototype.ngOnChanges = function (changes) {
        if ('markup' in changes) {
            this.markupMention = markupToRegExp(this.markup);
        }
    };
    NgHighlighterPatternDirective.prototype.match = function (value) {
        return this.markupMention ? this.markupMention.regEx.exec(value) : null;
    };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgHighlighterPatternDirective.prototype, "className", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgHighlighterPatternDirective.prototype, "markup", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], NgHighlighterPatternDirective.prototype, "markupReplace", void 0);
    NgHighlighterPatternDirective = __decorate([
        Directive({ exportAs: 'ngHighlighterPattern', selector: 'ng-highlighter-pattern' })
    ], NgHighlighterPatternDirective);
    return NgHighlighterPatternDirective;
}());

/**
 * The Highlighted Value
 */
var NgHighlightedValue = /** @class */ (function () {
    function NgHighlightedValue(content, type, rel) {
        if (type === void 0) { type = null; }
        if (rel === void 0) { rel = null; }
        /**
         * The type (or class name) associated with the highlighted item that was clicked.
         * @see NgHighlighterPatternDirective.className
         */
        this.type = null;
        /**
         * Optional. Arbitrary rel associated with the clicked highlighted element.
         * This is determined by how the highlighted item's content is formatted.
         */
        this.rel = null;
        this.content = content;
        this.type = type;
        this.rel = rel;
    }
    return NgHighlightedValue;
}());

/**
 * The Highlighter Component
 */
var NgHighlighterComponent = /** @class */ (function () {
    function NgHighlighterComponent(element, cdr) {
        this.element = element;
        this.cdr = cdr;
        /**
         * Event emitted when a highlighted item it clicked
         */
        this.itemClick = new EventEmitter();
        this.newLine = /\n/g;
        this.lines = [];
        this.highlightedElements = [];
    }
    NgHighlighterComponent.prototype.ngOnChanges = function (changes) {
        if ('text' in changes) {
            this.parseLines();
        }
    };
    NgHighlighterComponent.prototype.ngAfterContentInit = function () {
        if (this.text && this.patterns) {
            this.parseLines();
        }
    };
    NgHighlighterComponent.prototype.onItemClick = function (event) {
        var matchedElement = this.getMatchedElement(event);
        if (matchedElement) {
            event.preventDefault();
            event.stopPropagation();
            var content = matchedElement.element.innerText;
            var rel = matchedElement.element.getAttribute('rel') || null;
            var relElement = matchedElement.element.querySelector('[rel]');
            if (relElement) {
                rel = relElement.getAttribute('rel') || null;
            }
            this.itemClick.emit(new NgHighlightedValue(content, matchedElement.type, rel));
        }
    };
    NgHighlighterComponent.prototype.parseLines = function () {
        var _this = this;
        this.lines = this.text.split(this.newLine).map(function (line) { return _this.highlight(line); });
        this.cdr.detectChanges();
        this.collectHighlightedItems();
    };
    NgHighlighterComponent.prototype.highlight = function (line) {
        if (line.length === 0 || !this.patterns) {
            return '&nbsp;';
        }
        var tags = [];
        var match;
        this.patterns.forEach(function (pattern) {
            while ((match = pattern.match(line)) !== null) {
                tags.push({
                    type: pattern.className,
                    indices: { start: match.index, end: match.index + match[0].length },
                    formatter: pattern.format
                });
            }
        });
        var prevTags = [];
        var parts = [];
        __spread(tags).sort(function (tagA, tagB) { return tagA.indices.start - tagB.indices.start; }).forEach(function (tag) {
            var expectedLength = tag.indices.end - tag.indices.start;
            var contents = line.slice(tag.indices.start, tag.indices.end);
            if (contents.length === expectedLength) {
                var prevIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                var before = line.slice(prevIndex, tag.indices.start);
                parts.push(escapeHtml(before));
                parts.push("<span class=\"highlighted " + (tag.type || '') + "\" rel=\"" + tag.type + "\">" + tag.formatter(contents) + "</span>");
                prevTags.push(tag);
            }
        });
        var remainingStart = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
        var remaining = line.slice(remainingStart);
        parts.push(escapeHtml(remaining));
        parts.push('&nbsp;');
        return parts.join('');
    };
    NgHighlighterComponent.prototype.getMatchedElement = function (event) {
        var matched = this.highlightedElements.find(function (el) { return isCoordinateWithinRect(el.clientRect, event.clientX, event.clientY); });
        return matched ? matched : null;
    };
    NgHighlighterComponent.prototype.collectHighlightedItems = function () {
        this.highlightedElements = Array.from(this.element.nativeElement.getElementsByClassName('highlighted'))
            .map(function (element) {
            var type = element.getAttribute('rel') || null;
            return new Highlighted(element, type);
        });
    };
    NgHighlighterComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgHighlighterComponent.prototype, "text", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], NgHighlighterComponent.prototype, "itemClick", void 0);
    __decorate([
        ContentChildren(NgHighlighterPatternDirective),
        __metadata("design:type", QueryList)
    ], NgHighlighterComponent.prototype, "patterns", void 0);
    __decorate([
        HostListener('click', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [MouseEvent]),
        __metadata("design:returntype", void 0)
    ], NgHighlighterComponent.prototype, "onItemClick", null);
    NgHighlighterComponent = __decorate([
        Component({
            exportAs: 'ngHighlighter',
            selector: 'ng-highlighter',
            template: '<div *ngFor="let line of lines" [innerHTML]="line"></div>',
            preserveWhitespaces: false,
            encapsulation: ViewEncapsulation.None
        }),
        __metadata("design:paramtypes", [ElementRef, ChangeDetectorRef])
    ], NgHighlighterComponent);
    return NgHighlighterComponent;
}());

var Key;
(function (Key) {
    Key[Key["Backspace"] = 8] = "Backspace";
    Key[Key["Tab"] = 9] = "Tab";
    Key[Key["Enter"] = 13] = "Enter";
    Key[Key["Shift"] = 16] = "Shift";
    Key[Key["Escape"] = 27] = "Escape";
    Key[Key["Space"] = 32] = "Space";
    Key[Key["End"] = 35] = "End";
    Key[Key["Home"] = 36] = "Home";
    Key[Key["ArrowLeft"] = 37] = "ArrowLeft";
    Key[Key["ArrowUp"] = 38] = "ArrowUp";
    Key[Key["ArrowRight"] = 39] = "ArrowRight";
    Key[Key["ArrowDown"] = 40] = "ArrowDown";
})(Key || (Key = {}));

var NgMentionsListComponent = /** @class */ (function () {
    function NgMentionsListComponent() {
        this.activeIndex = -1;
        this.itemSelected = new EventEmitter();
        this.show = false;
        this.dropUp = false;
        this._top = 0;
        this._left = 0;
    }
    Object.defineProperty(NgMentionsListComponent.prototype, "selectedItem", {
        get: function () {
            return this.activeIndex >= 0 && this.items[this.activeIndex] !== undefined ? this.items[this.activeIndex] : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "top", {
        get: function () {
            return this._top + this.adjustTop + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "left", {
        get: function () {
            return this._left + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "noItems", {
        get: function () {
            return !Array.isArray(this.items) || this.items.length === 0;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsListComponent.prototype.ngOnInit = function () {
        if (!this.itemTemplate) {
            this.itemTemplate = this.defaultItemTemplate;
        }
    };
    NgMentionsListComponent.prototype.onItemClick = function (event, activeIndex, item) {
        event.preventDefault();
        this.activeIndex = activeIndex;
        this.itemSelected.emit(item);
    };
    NgMentionsListComponent.prototype.selectFirstItem = function () {
        this.activeIndex = 0;
        this.resetScroll();
    };
    NgMentionsListComponent.prototype.selectPreviousItem = function () {
        if (this.activeIndex > 0) {
            this.activeIndex--;
        }
        this.scrollToActiveItem();
    };
    NgMentionsListComponent.prototype.selectNextItem = function () {
        if (this.activeIndex < this.items.length - 1) {
            this.activeIndex++;
            this.scrollToActiveItem();
        }
    };
    NgMentionsListComponent.prototype.selectLastItem = function () {
        this.activeIndex = this.items.length > 0 ? this.items.length - 1 : 0;
        this.scrollToActiveItem();
    };
    NgMentionsListComponent.prototype.position = function () {
        var element = this.textAreaElement;
        var coords = getCaretCoordinates(element, element.selectionStart);
        this._top = coords.top;
        this._left = coords.left + element.offsetLeft;
        this.list.nativeElement.scrollTop = 0;
    };
    NgMentionsListComponent.prototype.resetScroll = function () {
        this.list.nativeElement.scrollTop = 0;
    };
    NgMentionsListComponent.prototype.transformItem = function (item) {
        return this.displayTransform(item) || item;
    };
    Object.defineProperty(NgMentionsListComponent.prototype, "adjustTop", {
        get: function () {
            var adjust = 0;
            if (!this.dropUp) {
                var computedFontSize = getElementStyle(this.textAreaElement, 'fontSize');
                adjust = parseInt(computedFontSize, 10) + this.textAreaElement.offsetTop;
            }
            return adjust;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsListComponent.prototype.scrollToActiveItem = function () {
        var element = this.list.nativeElement;
        if (this.activeIndex === 0) {
            element.scrollTop = 0;
        }
        else {
            var activeElement = element.querySelector('li.active');
            if (activeElement) {
                element.scrollTop = activeElement.offsetTop;
            }
        }
    };
    __decorate([
        ViewChild('defaultItemTemplate', { static: true }),
        __metadata("design:type", TemplateRef)
    ], NgMentionsListComponent.prototype, "defaultItemTemplate", void 0);
    __decorate([
        ViewChild('list', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgMentionsListComponent.prototype, "list", void 0);
    __decorate([
        HostBinding('class.show'),
        __metadata("design:type", Boolean)
    ], NgMentionsListComponent.prototype, "show", void 0);
    __decorate([
        HostBinding('class.drop-up'),
        __metadata("design:type", Boolean)
    ], NgMentionsListComponent.prototype, "dropUp", void 0);
    __decorate([
        HostBinding('style.top'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], NgMentionsListComponent.prototype, "top", null);
    __decorate([
        HostBinding('style.left'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], NgMentionsListComponent.prototype, "left", null);
    __decorate([
        HostBinding('class.no-items'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], NgMentionsListComponent.prototype, "noItems", null);
    NgMentionsListComponent = __decorate([
        Component({
            selector: 'mentions-list',
            template: "\n      <ng-template #defaultItemTemplate let-item=\"item\">\n          {{transformItem(item)}}\n      </ng-template>\n      <ul #list class=\"dropdown-menu scrollable-menu\">\n          <li *ngFor=\"let item of items; let i = index\" [class.active]=\"activeIndex === i\">\n              <a href class=\"dropdown-item\" (click)=\"onItemClick($event, i, item)\">\n                  <ng-template [ngTemplateOutlet]=\"itemTemplate\"\n                               [ngTemplateOutletContext]=\"{item:item,index:i}\"></ng-template>\n              </a>\n          </li>\n      </ul>\n  ",
            encapsulation: ViewEncapsulation.None,
            styles: ['mentions-list {position: absolute;display: none;}', 'mentions-list.drop-up {transform: translateY(-100%);}',
                'mentions-list.show {display: block;} mentions-list.no-items {display: none;}',
                'mentions-list .scrollable-menu {display: block;height: auto;max-height:300px;overflow:auto;}',
                'mentions-list li.active {background: #f7f7f9;}']
        })
    ], NgMentionsListComponent);
    return NgMentionsListComponent;
}());

/**
 * The Mentions Component
 */
var NgMentionsComponent = /** @class */ (function () {
    function NgMentionsComponent(element, componentResolver, viewContainer, changeDetector, ngZone) {
        this.element = element;
        this.componentResolver = componentResolver;
        this.viewContainer = viewContainer;
        this.changeDetector = changeDetector;
        this.ngZone = ngZone;
        /**
         * The character to trigger the mentions list when a user is typing in the input field
         */
        this.triggerChar = '@';
        /**
         * The markup used to format a mention in the model value
         */
        this.mentionMarkup = '@[__display__](__type__:__id__)';
        /**
         * Optional. When using a custom search (i.e. an API call), the internal searching capability should be disabled.
         */
        this.disableSearch = false;
        /**
         * Only used when internal search is not disabled. This limits the maximum number of items to display in the search
         * result list.
         */
        this.maxItems = -1;
        /**
         * Used to cause the search result list to display in a "drop up" fashion, instead of a typical dropdown.
         */
        this.dropUp = false;
        /**
         * If the supplied mentions are a list of objects, this is the name of the property used to display
         * the mention in the search result list and when formatting a mention in the displayed text.
         */
        this.displayName = 'display';
        /**
         * An event emitted, after the trigger character has been typed, with the user-entered search string.
         */
        this.search = new EventEmitter();
        this.valueChanges = new EventEmitter();
        this.stateChanges = new Subject();
        this.displayContent = '';
        this.lines = [];
        this.highlighterStyle = {};
        this.textAreaClassNames = {};
        this.mentions = [];
        this._value = '';
        this._rows = 1;
        this._columns = 20;
        this.stopSearch = false;
        this._destroyed = new Subject();
        this.newLine = /\n/g;
        this._errorState = false;
    }
    Object.defineProperty(NgMentionsComponent.prototype, "formClass", {
        /**
         * Classes for textarea
         */
        get: function () {
            return Object.keys(this.textAreaClassNames).join(' ');
        },
        set: function (classNames) {
            var _this = this;
            this.textAreaClassNames = {};
            Array.from(classNames.split(' ')).forEach(function (className) {
                _this.textAreaClassNames[className] = true;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this.parseLines(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (value) {
            this._disabled = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "rows", {
        /**
         * Number of rows for the textarea. Defaults to 1
         */
        get: function () {
            return this._rows;
        },
        set: function (value) {
            if (value !== null && typeof value !== 'undefined') {
                if (typeof value === 'string') {
                    value = parseInt(value, 10);
                }
                this._rows = Math.max(1, value);
                this.refreshStyles();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "columns", {
        /**
         * Number of columns for the textarea. Defaults to 1
         */
        get: function () {
            return this._columns;
        },
        set: function (value) {
            if (value !== null && typeof value !== 'undefined') {
                if (typeof value === 'string') {
                    value = parseInt(value, 10);
                }
                this._columns = Math.max(1, value);
                this.refreshStyles();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "mentionItems", {
        /**
         * The list of mentions to display, or filter, in the search result list.
         */
        set: function (value) {
            this.mentions = value;
            if (this.disableSearch && this.mentionsList) {
                this.mentionsList.items = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "readonly", {
        get: function () {
            return this.disabled ? 'readonly' : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "errorState", {
        get: function () {
            return this._errorState;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsComponent.prototype.ngOnInit = function () {
        this.parseMarkup();
    };
    NgMentionsComponent.prototype.ngOnChanges = function (changes) {
        if ('markup' in changes) {
            this.parseMarkup();
        }
    };
    NgMentionsComponent.prototype.ngAfterViewInit = function () {
        this.parseLines(this._value);
        this.refreshStyles();
    };
    NgMentionsComponent.prototype.ngAfterViewChecked = function () {
        this.refreshStyles();
    };
    NgMentionsComponent.prototype.ngOnDestroy = function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    NgMentionsComponent.prototype.onWindowResize = function () {
        this.refreshStyles();
    };
    NgMentionsComponent.prototype.onTextAreaScroll = function () {
        this.highlighterElement.nativeElement.scrollTop = this.textAreaInputElement.nativeElement.scrollTop;
    };
    NgMentionsComponent.prototype.onSelect = function (event) {
        this.selectionStart = event.target.selectionStart;
        this.selectionEnd = event.target.selectionEnd;
    };
    NgMentionsComponent.prototype.onChange = function (newPlainTextValue) {
        var _this = this;
        var value = this._value;
        var displayTransform = this.displayTransform.bind(this);
        var selectionStart = this.textAreaInputElement.nativeElement.selectionStart;
        var selectionEnd = this.textAreaInputElement.nativeElement.selectionEnd;
        var bounds = getBoundsOfMentionAtPosition(newPlainTextValue, this.markupSearch, selectionStart, displayTransform);
        if (bounds.start !== -1) {
            newPlainTextValue = newPlainTextValue.substring(0, bounds.start) + newPlainTextValue.substring(bounds.end);
        }
        var newValue = applyChangeToValue(value, this.markupSearch, newPlainTextValue, this.selectionStart, this.selectionEnd, selectionEnd, displayTransform);
        var startOfMention = findStartOfMentionInPlainText(value, this.markupSearch, selectionStart, displayTransform);
        if (startOfMention.start > -1 && this.selectionEnd > startOfMention.start) {
            selectionStart = startOfMention.start;
            selectionEnd = selectionStart;
        }
        this.selectionStart = Math.max(selectionStart, 0);
        this.selectionEnd = Math.max(selectionEnd, 0);
        this.parseLines(newValue);
        if (this.selectionEnd > 0) {
            setTimeout(function () { return setCaretPosition(_this.textAreaInputElement.nativeElement, _this.selectionEnd); });
        }
    };
    NgMentionsComponent.prototype.onKeyDown = function (event) {
        var caretPosition = getCaretPosition(this.textAreaInputElement.nativeElement);
        var characterPressed = event.key;
        var keyCode = event.which || event.keyCode;
        if (!characterPressed) {
            var characterCode = event.which || event.keyCode;
            characterPressed = String.fromCharCode(characterCode);
            if (!event.shiftKey && (characterCode >= 65 && characterCode <= 90)) {
                characterPressed = String.fromCharCode(characterCode + 32);
            }
        }
        if (keyCode === Key.Enter && event.wasSelection && caretPosition < this.startPos) {
            caretPosition = this.startNode.length;
            setCaretPosition(this.startNode, caretPosition);
        }
        var startOfMention = findStartOfMentionInPlainText(this._value, this.markupSearch, caretPosition, this.displayTransform.bind(this));
        if (characterPressed === this.triggerChar) {
            this.setupMentionsList(caretPosition);
        }
        else if (startOfMention.start === -1 && this.startPos >= 0) {
            if (caretPosition <= this.startPos) {
                this.mentionsList.show = false;
                this.startPos = -1;
            }
            else if (keyCode !== Key.Shift && !event.shiftKey && !event.metaKey && !event.altKey && !event.ctrlKey &&
                caretPosition > this.startPos) {
                this.handleKeyDown(event, caretPosition, characterPressed);
            }
        }
        else {
            this.onSelect({ target: this.textAreaInputElement.nativeElement });
        }
    };
    NgMentionsComponent.prototype.onBlur = function (event) {
        if (event instanceof FocusEvent && event.relatedTarget) {
            var element = event.relatedTarget;
            if (element.classList.contains('dropdown-item')) {
                return;
            }
        }
        this.stopEvent(event);
        if (this.mentionsList) {
            this.mentionsList.show = false;
        }
    };
    NgMentionsComponent.prototype.isPartMention = function (part) {
        return typeof part.contents !== 'undefined';
    };
    NgMentionsComponent.prototype.formatMention = function (mention) {
        return this._formatMention(mention.contents);
    };
    NgMentionsComponent.prototype.displayTransform = function () {
        var _ = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            _[_i] = arguments[_i];
        }
        var replaceIndex = this.markupSearch.groups[this.displayName];
        return _[replaceIndex];
    };
    NgMentionsComponent.prototype._formatMention = function (contents) {
        var replaceValue = "$" + this.displayName;
        var result = contents.replace(this.markupSearch.regEx, replaceValue), replaceIndex;
        if (result === replaceValue) {
            replaceIndex = "$" + this.markupSearch.groups[this.displayName];
            result = contents.replace(this.markupSearch.regEx, replaceIndex);
        }
        return result;
    };
    NgMentionsComponent.prototype.stopEvent = function (event) {
        if (event.preventDefault) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();
        }
    };
    NgMentionsComponent.prototype.setupMentionsList = function (caretPosition) {
        this.startPos = caretPosition;
        this.startNode = window.getSelection().anchorNode;
        this.searchString = '';
        this.stopSearch = false;
        this.showMentionsList();
        this.updateMentionsList();
    };
    NgMentionsComponent.prototype.handleKeyDown = function (event, caretPosition, characterPressed) {
        var _this = this;
        var keyCode = event.which || event.keyCode;
        if (keyCode === Key.Space) {
            this.startPos = -1;
        }
        else if (keyCode === Key.Backspace && caretPosition > 0) {
            caretPosition--;
            if (caretPosition === this.startPos) {
                this.stopSearch = true;
            }
            this.mentionsList.show = !this.stopSearch;
        }
        else if (this.mentionsList.show) {
            if (keyCode === Key.Tab || keyCode === Key.Enter) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                var value = this._value;
                var start = mapPlainTextIndex(value, this.markupSearch, this.startPos, false, this.displayTransform.bind(this));
                var item = event.item || this.mentionsList.selectedItem;
                var newValue = replacePlaceholders(item, this.markupSearch);
                var newDisplayValue = this._formatMention(newValue);
                caretPosition = this.startPos + newDisplayValue.length;
                var searchString = this.searchString || '';
                value = value.substring(0, start) + newValue + value.substring(start + searchString.length + 1, value.length);
                this.parseLines(value);
                this.startPos = -1;
                this.searchString = '';
                this.stopSearch = true;
                this.mentionsList.show = false;
                this.changeDetector.detectChanges();
                setTimeout(function () {
                    setCaretPosition(_this.textAreaInputElement.nativeElement, caretPosition);
                    _this.onSelect({ target: _this.textAreaInputElement.nativeElement });
                });
                return;
            }
            else if (keyCode === Key.Escape) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                this.stopSearch = true;
                return;
            }
            else if (keyCode === Key.ArrowDown) {
                this.stopEvent(event);
                this.mentionsList.selectNextItem();
                return;
            }
            else if (keyCode === Key.ArrowUp) {
                this.stopEvent(event);
                this.mentionsList.selectPreviousItem();
                return;
            }
            else if (keyCode === Key.Home) {
                this.stopEvent(event);
                this.mentionsList.selectFirstItem();
                return;
            }
            else if (keyCode === Key.End) {
                this.stopEvent(event);
                this.mentionsList.selectLastItem();
                return;
            }
        }
        if (keyCode === Key.ArrowLeft || keyCode === Key.ArrowRight || keyCode === Key.Home || keyCode === Key.End) {
            this.onSelect(event);
            return;
        }
        var mention = this.displayContent.substring(this.startPos + 1, caretPosition);
        if (keyCode !== Key.Backspace) {
            mention += characterPressed;
        }
        this.searchString = mention || '';
        this.updateMentionsList();
    };
    NgMentionsComponent.prototype.getDisplayValue = function (item) {
        if (typeof item === 'string') {
            return item;
        }
        else if (item[this.displayName] !== undefined) {
            return item[this.displayName];
        }
        return null;
    };
    NgMentionsComponent.prototype.showMentionsList = function () {
        var _this = this;
        if (!this.mentionsList) {
            var componentFactory = this.componentResolver.resolveComponentFactory(NgMentionsListComponent);
            var componentRef = this.viewContainer.createComponent(componentFactory);
            this.mentionsList = componentRef.instance;
            this.mentionsList.itemTemplate = this.mentionListTemplate;
            this.mentionsList.displayTransform = this.displayTransform.bind(this);
            this.mentionsList.itemSelected.subscribe(function (item) {
                _this.textAreaInputElement.nativeElement.focus();
                var fakeEvent = { keyCode: Key.Enter, wasSelection: true, item: item };
                _this.onKeyDown(fakeEvent);
            });
            this.mentionsList.displayTransform = this.getDisplayValue.bind(this);
        }
        this.mentionsList.textAreaElement = this.textAreaInputElement.nativeElement;
        this.mentionsList.show = true;
        this.mentionsList.dropUp = this.dropUp;
        this.mentionsList.activeIndex = 0;
        this.mentionsList.position();
        this.ngZone.run(function () { return _this.mentionsList.resetScroll(); });
    };
    NgMentionsComponent.prototype.updateMentionsList = function () {
        var _this = this;
        if (!this.disableSearch) {
            var items = Array.from(this.mentions);
            if (this.searchString) {
                var searchString = this.searchString.toLowerCase(), searchRegEx_1 = new RegExp(escapeRegExp(searchString), 'i');
                items = items.filter(function (item) {
                    var value = _this.getDisplayValue(item);
                    return value !== null && searchRegEx_1.test(value);
                });
                if (this.maxItems > 0) {
                    items = items.slice(0, this.maxItems);
                }
            }
            this.mentionsList.items = items;
        }
        else {
            this.search.emit(this.searchString);
        }
    };
    NgMentionsComponent.prototype.parseMarkup = function () {
        if (this.mentionMarkup.length === 0 || this.mentionMarkup[0] !== this.triggerChar) {
            throw new Error("ng-mentions markup pattern must start with the trigger character \"" + this.triggerChar + "\"");
        }
        this.markupSearch = markupToRegExp(this.mentionMarkup);
    };
    NgMentionsComponent.prototype.parseLines = function (value) {
        var _this = this;
        if (value === void 0) { value = ''; }
        if (value !== this._value) {
            value = value || '';
            var lines = value.split(this.newLine).map(function (line) { return _this.formatMentions(line); });
            var displayContent = lines.map(function (line) { return line.content; }).join('\n');
            if (this.displayContent !== displayContent) {
                this.lines = lines;
                this.displayContent = displayContent;
                this.triggerChange(value);
            }
        }
    };
    NgMentionsComponent.prototype.formatMentions = function (line) {
        var _this = this;
        var lineObj = { originalContent: line, content: line, parts: [] };
        if (line.length === 0) {
            return lineObj;
        }
        var mention;
        var tags = [];
        var regEx = this.markupSearch.regEx;
        regEx.lastIndex = 0;
        while ((mention = regEx.exec(line)) !== null) {
            tags.push({ indices: { start: mention.index, end: mention.index + mention[0].length } });
        }
        var prevTags = [];
        var content = '';
        __spread(tags).sort(function (tagA, tagB) { return tagA.indices.start - tagB.indices.start; }).forEach(function (tag) {
            var expectedLength = tag.indices.end - tag.indices.start;
            var contents = line.slice(tag.indices.start, tag.indices.end);
            if (contents.length === expectedLength) {
                var prevIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                var before = line.slice(prevIndex, tag.indices.start);
                var partMention = { contents: contents, tag: tag };
                lineObj.parts.push(before);
                lineObj.parts.push(partMention);
                prevTags.push(tag);
                content += before + _this.formatMention(partMention);
            }
        });
        var remainingStart = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
        var remaining = line.slice(remainingStart);
        lineObj.parts.push(remaining);
        content += remaining;
        lineObj.content = content;
        return lineObj;
    };
    NgMentionsComponent.prototype.refreshStyles = function () {
        var _this = this;
        var element = this.textAreaInputElement.nativeElement;
        var computedStyle = getComputedStyle(element);
        this.highlighterStyle = {};
        styleProperties.forEach(function (prop) {
            _this.highlighterStyle[prop] = computedStyle[prop];
        });
        this.changeDetector.detectChanges();
    };
    NgMentionsComponent.prototype.triggerChange = function (value) {
        this._value = value;
        this.valueChanges.emit(this._value);
        this.changeDetector.detectChanges();
    };
    NgMentionsComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ComponentFactoryResolver },
        { type: ViewContainerRef },
        { type: ChangeDetectorRef },
        { type: NgZone }
    ]; };
    __decorate([
        Input('triggerChar'),
        __metadata("design:type", String)
    ], NgMentionsComponent.prototype, "triggerChar", void 0);
    __decorate([
        Input('markup'),
        __metadata("design:type", String)
    ], NgMentionsComponent.prototype, "mentionMarkup", void 0);
    __decorate([
        Input('disableSearch'),
        __metadata("design:type", Boolean)
    ], NgMentionsComponent.prototype, "disableSearch", void 0);
    __decorate([
        Input('maxItems'),
        __metadata("design:type", Number)
    ], NgMentionsComponent.prototype, "maxItems", void 0);
    __decorate([
        Input('dropUp'),
        __metadata("design:type", Boolean)
    ], NgMentionsComponent.prototype, "dropUp", void 0);
    __decorate([
        Input('displayName'),
        __metadata("design:type", String)
    ], NgMentionsComponent.prototype, "displayName", void 0);
    __decorate([
        Input('formClass'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgMentionsComponent.prototype, "formClass", null);
    __decorate([
        Input('placeholder'),
        __metadata("design:type", String)
    ], NgMentionsComponent.prototype, "placeholder", void 0);
    __decorate([
        Input('value'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgMentionsComponent.prototype, "value", null);
    __decorate([
        Input('required'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], NgMentionsComponent.prototype, "required", null);
    __decorate([
        Input('disabled'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], NgMentionsComponent.prototype, "disabled", null);
    __decorate([
        Input('rows'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], NgMentionsComponent.prototype, "rows", null);
    __decorate([
        Input('cols'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], NgMentionsComponent.prototype, "columns", null);
    __decorate([
        Input('mentions'),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], NgMentionsComponent.prototype, "mentionItems", null);
    __decorate([
        Output('search'),
        __metadata("design:type", EventEmitter)
    ], NgMentionsComponent.prototype, "search", void 0);
    __decorate([
        Output('valueChanges'),
        __metadata("design:type", EventEmitter)
    ], NgMentionsComponent.prototype, "valueChanges", void 0);
    __decorate([
        Output('stateChanges'),
        __metadata("design:type", Subject)
    ], NgMentionsComponent.prototype, "stateChanges", void 0);
    __decorate([
        ContentChild(TemplateRef, { static: true }),
        __metadata("design:type", TemplateRef)
    ], NgMentionsComponent.prototype, "mentionListTemplate", void 0);
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgMentionsComponent.prototype, "textAreaInputElement", void 0);
    __decorate([
        ViewChild('highlighter', { static: true }),
        __metadata("design:type", ElementRef)
    ], NgMentionsComponent.prototype, "highlighterElement", void 0);
    __decorate([
        HostListener('window:resize'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], NgMentionsComponent.prototype, "onWindowResize", null);
    NgMentionsComponent = __decorate([
        Component({
            exportAs: 'ngMentions',
            selector: 'ng-mentions',
            template: "\n      <div #highlighter class=\"highlighter\" [ngClass]=\"textAreaClassNames\" [attr.readonly]=\"readonly\"\n           [ngStyle]=\"highlighterStyle\">\n          <div *ngFor=\"let line of lines\">\n              <ng-container *ngFor=\"let part of line.parts\">\n                  <highlighted *ngIf=\"isPartMention(part)\" [tag]=\"part.tag\">{{formatMention(part)}}</highlighted>\n                  <ng-container *ngIf=\"!isPartMention(part)\">{{part}}</ng-container>\n              </ng-container>\n              &nbsp;\n          </div>\n      </div>\n      <textarea\n        #input\n        [rows]=\"rows\"\n        [cols]=\"columns\"\n        [ngModel]=\"displayContent\"\n        [ngClass]=\"textAreaClassNames\"\n        (keydown)=\"onKeyDown($event)\"\n        (blur)=\"onBlur($event)\"\n        (select)=\"onSelect($event)\"\n        (mouseup)=\"onSelect($event)\"\n        (ngModelChange)=\"onChange($event)\"\n        (scroll)=\"onTextAreaScroll()\"\n        [disabled]=\"disabled\"\n        [required]=\"required\"\n        [placeholder]=\"placeholder\"\n      ></textarea>\n  ",
            preserveWhitespaces: false,
            encapsulation: ViewEncapsulation.None,
            styles: ['ng-mentions {position: relative; display: inline-block;}',
                'ng-mentions textarea {position:relative; background-color: transparent !important;}', "ng-mentions .highlighter {\n        position: absolute;\n        top:      0;\n        left:     0;\n        right:    0;\n        bottom:   0;\n        color:    transparent;\n        overflow: hidden !important;\n    }",
                "ng-mentions highlighted {\n        display:          inline;\n        border-radius:    3px;\n        padding:          1px;\n        margin:           -1px;\n        overflow-wrap:    break-word;\n        background-color: lightblue;\n    }"]
        }),
        __metadata("design:paramtypes", [ElementRef, ComponentFactoryResolver,
            ViewContainerRef, ChangeDetectorRef, NgZone])
    ], NgMentionsComponent);
    return NgMentionsComponent;
}());

/**
 * The NgMentionsAccessorDirective directive is used to indicate the input element.
 */
var NgMentionsAccessorDirective = /** @class */ (function () {
    function NgMentionsAccessorDirective(element, host) {
        this.element = element;
        this.host = host;
        this._destroyed = new Subject();
    }
    NgMentionsAccessorDirective_1 = NgMentionsAccessorDirective;
    NgMentionsAccessorDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.host.valueChanges.pipe(takeUntil(this._destroyed)).subscribe(function (value) { return _this.onChange(value); });
    };
    NgMentionsAccessorDirective.prototype.ngOnDestroy = function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    NgMentionsAccessorDirective.prototype.registerOnChange = function (fn) {
        this._onChange = fn;
    };
    NgMentionsAccessorDirective.prototype.registerOnTouched = function (fn) {
        this._onTouch = fn;
    };
    NgMentionsAccessorDirective.prototype.setDisabledState = function (isDisabled) {
        this.host.disabled = isDisabled;
    };
    NgMentionsAccessorDirective.prototype.writeValue = function (value) {
        if (typeof value === 'string' || value === null) {
            this.host.value = value;
        }
    };
    NgMentionsAccessorDirective.prototype.onChange = function (value) {
        if (this._onChange && typeof value !== 'object') {
            this._onChange(value);
        }
    };
    NgMentionsAccessorDirective.prototype.onTouched = function () {
        if (this._onTouch) {
            this._onTouch();
        }
    };
    var NgMentionsAccessorDirective_1;
    NgMentionsAccessorDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgMentionsComponent }
    ]; };
    NgMentionsAccessorDirective = NgMentionsAccessorDirective_1 = __decorate([
        Directive({
            exportAs: 'ngMentions',
            selector: 'ng-mentions',
            host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
            providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return NgMentionsAccessorDirective_1; }), multi: true }]
        }),
        __metadata("design:paramtypes", [ElementRef, NgMentionsComponent])
    ], NgMentionsAccessorDirective);
    return NgMentionsAccessorDirective;
}());

var HighlightedDirective = /** @class */ (function () {
    function HighlightedDirective() {
    }
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], HighlightedDirective.prototype, "tag", void 0);
    HighlightedDirective = __decorate([
        Directive({ selector: 'highlighted' })
    ], HighlightedDirective);
    return HighlightedDirective;
}());

var EXPORT_DIRECTIVES = [
    NgMentionsComponent,
    NgMentionsAccessorDirective,
    NgHighlighterComponent,
    NgHighlighterPatternDirective,
];
var DECLARATIONS = [
    NgMentionsComponent, NgMentionsAccessorDirective, NgMentionsListComponent, HighlightedDirective,
    NgHighlighterComponent, NgHighlighterPatternDirective
];
var NgMentionsModule = /** @class */ (function () {
    function NgMentionsModule() {
    }
    NgMentionsModule = __decorate([
        NgModule({
            imports: [CommonModule, FormsModule],
            exports: EXPORT_DIRECTIVES,
            declarations: DECLARATIONS,
            entryComponents: [NgMentionsListComponent]
        })
    ], NgMentionsModule);
    return NgMentionsModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NgHighlighterComponent, NgMentionsAccessorDirective, NgMentionsComponent, NgMentionsModule, NgHighlighterPatternDirective as ɵa, NgMentionsListComponent as ɵb, HighlightedDirective as ɵc };
//# sourceMappingURL=ng-mentions.js.map
